/**
*	@author: Thomas Brandého
*/

#include "input.h"


#include <iostream>
#include <SFML/Window.hpp>
#include <list>

Input::Input(sf::Window *w) :
    m_window(w),
    m_closeRequested(false),
    m_mouseGrabbed(false)
{
    m_heldkeys = std::vector<sf::Keyboard::Key>();
    m_actions = std::vector<Action>();
    m_window->setKeyRepeatEnabled(false);
}

void Input::setKeysMap(IKeysMap km){
    m_keysmap = km;
}

void Input::updateEvents(){
	sf::Event event;
	KeyBindings kb;
	char c;

	/* reset variables */
    m_closeRequested = false;
    m_hasBeenResized = false;
    m_delta_vertical_scroll = 0;
    m_buffer.clear();

    if(m_mouseGrabbed)
        m_last_mouse_position = sf::Vector2i(m_window->getSize())/2;
    else
        m_last_mouse_position = m_mouse_position;

	/* global affectation */
    kb = m_keybindings[m_current_context];
    m_actions.clear();

    Action action;
    /*
    sf::Joystick::X;
    sf::Joystick::Y;
    sf::Joystick::Z;
    sf::Joystick::R;
    sf::Joystick::U;
    sf::Joystick::V;
    sf::Joystick::PovX;
    sf::Joystick::PovY;
    */

    /* event-parsing loop */
    while(m_window->pollEvent(event))
    {
		switch(event.type){
        case sf::Event::Closed:
            m_closeRequested = true;
            break;
        case sf::Event::Resized:
            m_hasBeenResized = true;
            if(m_mouseGrabbed)
                m_last_mouse_position = sf::Vector2i(m_window->getSize())/2;
            break;
        case sf::Event::LostFocus:
        case sf::Event::GainedFocus:
            //Nothing to add here because the window already keep the state of the focus
            break;
        case sf::Event::TextEntered:
            c = (char) event.text.unicode;
            sf::Utf32::encodeAnsi(event.text.unicode,std::back_inserter(m_buffer));
            break;
        case sf::Event::KeyPressed:
            action = kb.getPressedAction(input::KEYBOARD,event.key.code);
            if(!action.isNull())
                m_actions.push_back(action);
//            m_heldkeys.push_back(event.key.code);
            break;
        case sf::Event::KeyReleased:
            action = kb.getReleasedAction(input::KEYBOARD,event.key.code);
            if(!action.isNull())
                m_actions.push_back(action);
//            releaseHeldKeys(event.key.code);
            break;
        case sf::Event::MouseWheelScrolled:
            if (event.mouseWheelScroll.wheel == sf::Mouse::VerticalWheel)
                m_delta_vertical_scroll = event.mouseWheelScroll.delta;
            break;
        case sf::Event::MouseButtonPressed:
            action = kb.getPressedAction(input::MOUSE,event.mouseButton.button);
            if(!action.isNull())
                m_actions.push_back(action);
//            m_heldkeys.push_back((sf::Keyboard::Key) (event.mouseButton.button));
            break;
        case sf::Event::MouseButtonReleased:
            action = kb.getReleasedAction(input::MOUSE,event.mouseButton.button);
            if(!action.isNull())
                m_actions.push_back(action);
//            releaseHeldKeys((sf::Keyboard::Key) (event.mouseButton.button));
            break;
        case sf::Event::MouseMoved:
            m_mouse_position = sf::Mouse::getPosition(*m_window);
            break;
        case sf::Event::MouseEntered:
            // action MouseEntered
            break;
        case sf::Event::MouseLeft:
            //action MouseLeft
            break;
        case sf::Event::JoystickButtonPressed:
            action = kb.getPressedAction(input::CONTROLLER,event.joystickButton.button); // + idjoy * joystick::buttoncount
            if(!action.isNull()){
                action.controller_id = event.joystickButton.joystickId;
                m_actions.push_back(action);
            }
            m_heldkeys.push_back((sf::Keyboard::Key) (event.joystickButton.button));
            break;
        case sf::Event::JoystickButtonReleased:
            action = kb.getReleasedAction(input::CONTROLLER, event.joystickButton.button); // + idjoy * joystick::buttoncount
            if(!action.isNull()){
                action.controller_id = event.joystickButton.joystickId;
                m_actions.push_back(action);
            }
            releaseHeldKeys((sf::Keyboard::Key) (event.joystickButton.button));
            break;
        case sf::Event::JoystickMoved:
            // i'll handle that later
            std::cout << "Joystick axis " << event.joystickMove.axis << " of controller " << event.joystickMove.joystickId << " moved at" << event.joystickMove.position << std::endl;
            break;
        case sf::Event::JoystickConnected:
        {
            sf::Joystick::Identification id = sf::Joystick::getIdentification(event.joystickConnect.joystickId);
            std::cout << "Joystick "<< event.joystickConnect.joystickId << " connected :" << id.name.toAnsiString() << std::endl;
            break;
        }

        case sf::Event::JoystickDisconnected:
        {
            sf::Joystick::Identification id = sf::Joystick::getIdentification(event.joystickConnect.joystickId);
            std::cout << "Joystick "<< event.joystickConnect.joystickId << " disconnected :" << id.name.toAnsiString() << std::endl;
            break;
        }
        }
	}
    if(m_mouseGrabbed)
        sf::Mouse::setPosition(m_last_mouse_position, *m_window);

    for (auto key: m_heldkeys)
    {
//        m_actions.push_back(kb.getHoldAction(key));
	}
}

std::vector<Action> Input::getActions()
{
    return m_actions;
}

/* context-related functions */
void Input::updateKeyBindings(){
    m_keybindings.clear();
    for (auto iter= m_contexts.begin(); iter != m_contexts.end(); ++iter)
        m_keybindings[iter->getName()]= KeyBindings(*iter,m_keysmap);
}

/* window-related function */

bool Input::isCloseRequested() const
{
    return m_closeRequested;
}

bool Input::isResized() const
{
    return m_hasBeenResized;
}

/* keyboard-related functions */

bool Input::isKeyPressed(int key) const
{
	return sf::Keyboard::isKeyPressed((sf::Keyboard::Key) key);
}

void Input::releaseHeldKeys(sf::Keyboard::Key keycode){
    for(auto iter = m_heldkeys.begin();iter < m_heldkeys.end();)
        iter = *iter == keycode ? m_heldkeys.erase(iter) : iter+1;
}

/* mouse-related functions */

glm::vec2 Input::getPosition() const
{
    return glm::vec2(m_mouse_position.x,m_mouse_position.y);
}

glm::vec2 Input::getDeltaPosition() const
{
    sf::Vector2i delta = m_mouse_position - m_last_mouse_position;
    return glm::vec2(delta.x,delta.y);
}

float Input::getDeltaVerticalScroll() const
{
    return m_delta_vertical_scroll;
}

void Input::setMouseGrabbed(bool isGrabbed)
{
    m_mouseGrabbed = isGrabbed;
    if(isGrabbed)
        m_last_mouse_position = sf::Vector2i(m_window->getSize())/2;
}

float getAxisPosition(int device_id, int axis_id)
{

}

std::wstring Input::getText()
{
    return m_buffer;
}

/* ------ */

void Input::test()
{
/*    KeyBindings	kb = m_keybindings[m_current_context];
	int action;
    action = kb.getPressedAction(sf::Keyboard::I);
	if (action != NO_ACTION)
		std::cerr << action << std::endl;
	action = kb.getPressedAction(sf::Keyboard::O);
	if (action != NO_ACTION)
        std::cerr << action << std::endl;*/
}
